@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <X11/keysym.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gio/gio.h>
 #include <matekbd-desktop-config.h>
 #include <matekbd-config-private.h>
@@ -68,13 +68,13 @@ static gboolean
 	pv = variant_ids;
 	total_layouts = g_strv_length ((char **) layout_ids);
 	sld = *short_layout_descriptions =
-	    g_new0 (char *, total_layouts + 1);
+	    g_new0 (gchar *, total_layouts + 1);
 	lld = *long_layout_descriptions =
-	    g_new0 (char *, total_layouts + 1);
+	    g_new0 (gchar *, total_layouts + 1);
 	svd = *short_variant_descriptions =
-	    g_new0 (char *, total_layouts + 1);
+	    g_new0 (gchar *, total_layouts + 1);
 	lvd = *long_variant_descriptions =
-	    g_new0 (char *, total_layouts + 1);
+	    g_new0 (gchar *, total_layouts + 1);
 
 	while (pl != NULL && *pl != NULL) {
 
@@ -239,6 +239,10 @@ matekbd_desktop_config_restore_group (MatekbdDesktopConfig * config)
 	xkl_engine_lock_group (config->engine, group);
 }
 
+/**
+ * matekbd_desktop_config_start_listen:
+ * @func: (scope notified): a function to call when settings are changed
+ */
 void
 matekbd_desktop_config_start_listen (MatekbdDesktopConfig * config,
 				  GCallback func,
@@ -272,7 +276,7 @@ matekbd_desktop_config_load_group_descriptions (MatekbdDesktopConfig
 {
 	gchar **sld, **lld, **svd, **lvd;
 	gchar **psld, **plld, **plvd;
-	gchar **psgn, **pfgn;
+	gchar **psgn, **pfgn, **psvd;
 	gint total_descriptions;
 
 	if (!matekbd_desktop_config_get_lv_descriptions
@@ -291,8 +295,11 @@ matekbd_desktop_config_load_group_descriptions (MatekbdDesktopConfig
 	plld = lld;
 	psld = sld;
 	plvd = lvd;
+	psvd = svd;
 	while (plld != NULL && *plld != NULL) {
-		*psgn++ = g_strdup (*psld++);
+		gchar *sd = (*psvd[0] == '\0') ? *psld : *psvd;
+		psld++, psvd++;
+		*psgn++ = g_strdup (sd);
 		*pfgn++ = g_strdup (matekbd_keyboard_config_format_full_layout
 				    (*plld++, *plvd++));
 	}
