@@ -16,13 +16,14 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#include <config.h>
 
 #include <memory.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <matekbd-indicator.h>
 #include <matekbd-indicator-marshal.h>
@@ -510,7 +511,7 @@ matekbd_indicator_load_group_names (const gchar ** layout_ids,
 		gint i, total_groups =
 		    xkl_engine_get_num_groups (globals.engine);
 		globals.full_group_names =
-		    g_new0 (char *, total_groups + 1);
+		    g_new0 (gchar *, total_groups + 1);
 
 		if (xkl_engine_get_features (globals.engine) &
 		    XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
@@ -870,6 +871,10 @@ matekbd_indicator_get_xkl_engine ()
 	return globals.engine;
 }
 
+/**
+ * matekbd_indicator_get_group_names:
+ * Returns: (transfer none) (array zero-terminated=1): List of group names
+ */
 gchar **
 matekbd_indicator_get_group_names ()
 {
