@@ -21,15 +21,12 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#if GTK_CHECK_VERSION (3, 0, 0)
-#include <gdk/gdkkeysyms-compat.h>
-#endif
 #include <X11/XKBlib.h>
 #include <X11/extensions/XKBgeom.h>
 #include <stdlib.h>
 #include <memory.h>
 #include <math.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <libxklavier/xklavier.h>
 
 #include <matekbd-keyboard-drawing.h>
@@ -621,6 +618,21 @@ find_keycode (MatekbdKeyboardDrawing * drawing, gchar * key_name)
 	return INVALID_KEYCODE;
 }
 
+static void
+set_markup (MatekbdKeyboardDrawingRenderContext * context, gchar *txt)
+{
+	PangoLayout *layout = context->layout;
+	txt = strcmp ("<", txt) ? txt : "&lt;";
+	txt = strcmp ("&", txt) ? txt : "&amp;";
+	if (g_utf8_strlen (txt, -1) > 1) {
+		gchar* buf =
+		    g_strdup_printf ("<span size=\"x-small\">%s</span>", txt);
+		pango_layout_set_markup (layout, buf, -1);
+		g_free (buf);
+	} else {
+		pango_layout_set_markup (layout, txt, -1);
+	}
+}
 
 static void
 set_key_label_in_layout (MatekbdKeyboardDrawingRenderContext * context,
@@ -628,164 +640,163 @@ set_key_label_in_layout (MatekbdKeyboardDrawingRenderContext * context,
 {
 	gchar buf[5];
 	gunichar uc;
-	PangoLayout *layout = context->layout;
 
 	switch (keyval) {
-	case GDK_Scroll_Lock:
-		pango_layout_set_text (layout, "Scroll\nLock", -1);
+	case GDK_KEY_Scroll_Lock:
+		set_markup (context, "Scroll\nLock");
 		break;
 
-	case GDK_space:
-		pango_layout_set_text (layout, "", -1);
+	case GDK_KEY_space:
+		set_markup (context, "");
 		break;
 
-	case GDK_Sys_Req:
-		pango_layout_set_text (layout, "Sys Rq", -1);
+	case GDK_KEY_Sys_Req:
+		set_markup (context, "Sys Rq");
 		break;
 
-	case GDK_Page_Up:
-		pango_layout_set_text (layout, "Page\nUp", -1);
+	case GDK_KEY_Page_Up:
+		set_markup (context, "Page\nUp");
 		break;
 
-	case GDK_Page_Down:
-		pango_layout_set_text (layout, "Page\nDown", -1);
+	case GDK_KEY_Page_Down:
+		set_markup (context, "Page\nDown");
 		break;
 
-	case GDK_Num_Lock:
-		pango_layout_set_text (layout, "Num\nLock", -1);
+	case GDK_KEY_Num_Lock:
+		set_markup (context, "Num\nLock");
 		break;
 
-	case GDK_KP_Page_Up:
-		pango_layout_set_text (layout, "Pg Up", -1);
+	case GDK_KEY_KP_Page_Up:
+		set_markup (context, "Pg Up");
 		break;
 
-	case GDK_KP_Page_Down:
-		pango_layout_set_text (layout, "Pg Dn", -1);
+	case GDK_KEY_KP_Page_Down:
+		set_markup (context, "Pg Dn");
 		break;
 
-	case GDK_KP_Home:
-		pango_layout_set_text (layout, "Home", -1);
+	case GDK_KEY_KP_Home:
+		set_markup (context, "Home");
 		break;
 
-	case GDK_KP_Left:
-		pango_layout_set_text (layout, "Left", -1);
+	case GDK_KEY_KP_Left:
+		set_markup (context, "Left");
 		break;
 
-	case GDK_KP_End:
-		pango_layout_set_text (layout, "End", -1);
+	case GDK_KEY_KP_End:
+		set_markup (context, "End");
 		break;
 
-	case GDK_KP_Up:
-		pango_layout_set_text (layout, "Up", -1);
+	case GDK_KEY_KP_Up:
+		set_markup (context, "Up");
 		break;
 
-	case GDK_KP_Begin:
-		pango_layout_set_text (layout, "Begin", -1);
+	case GDK_KEY_KP_Begin:
+		set_markup (context, "Begin");
 		break;
 
-	case GDK_KP_Right:
-		pango_layout_set_text (layout, "Right", -1);
+	case GDK_KEY_KP_Right:
+		set_markup (context, "Right");
 		break;
 
-	case GDK_KP_Enter:
-		pango_layout_set_text (layout, "Enter", -1);
+	case GDK_KEY_KP_Enter:
+		set_markup (context, "Enter");
 		break;
 
-	case GDK_KP_Down:
-		pango_layout_set_text (layout, "Down", -1);
+	case GDK_KEY_KP_Down:
+		set_markup (context, "Down");
 		break;
 
-	case GDK_KP_Insert:
-		pango_layout_set_text (layout, "Ins", -1);
+	case GDK_KEY_KP_Insert:
+		set_markup (context, "Ins");
 		break;
 
-	case GDK_KP_Delete:
-		pango_layout_set_text (layout, "Del", -1);
+	case GDK_KEY_KP_Delete:
+		set_markup (context, "Del");
 		break;
 
-	case GDK_dead_grave:
-		pango_layout_set_text (layout, "ˋ", -1);
+	case GDK_KEY_dead_grave:
+		set_markup (context, "ˋ");
 		break;
 
-	case GDK_dead_acute:
-		pango_layout_set_text (layout, "ˊ", -1);
+	case GDK_KEY_dead_acute:
+		set_markup (context, "ˊ");
 		break;
 
-	case GDK_dead_circumflex:
-		pango_layout_set_text (layout, "ˆ", -1);
+	case GDK_KEY_dead_circumflex:
+		set_markup (context, "ˆ");
 		break;
 
-	case GDK_dead_tilde:
-		pango_layout_set_text (layout, "~", -1);
+	case GDK_KEY_dead_tilde:
+		set_markup (context, "~");
 		break;
 
-	case GDK_dead_macron:
-		pango_layout_set_text (layout, "ˉ", -1);
+	case GDK_KEY_dead_macron:
+		set_markup (context, "ˉ");
 		break;
 
-	case GDK_dead_breve:
-		pango_layout_set_text (layout, "˘", -1);
+	case GDK_KEY_dead_breve:
+		set_markup (context, "˘");
 		break;
 
-	case GDK_dead_abovedot:
-		pango_layout_set_text (layout, "˙", -1);
+	case GDK_KEY_dead_abovedot:
+		set_markup (context, "˙");
 		break;
 
-	case GDK_dead_diaeresis:
-		pango_layout_set_text (layout, "¨", -1);
+	case GDK_KEY_dead_diaeresis:
+		set_markup (context, "¨");
 		break;
 
-	case GDK_dead_abovering:
-		pango_layout_set_text (layout, "˚", -1);
+	case GDK_KEY_dead_abovering:
+		set_markup (context, "˚");
 		break;
 
-	case GDK_dead_doubleacute:
-		pango_layout_set_text (layout, "˝", -1);
+	case GDK_KEY_dead_doubleacute:
+		set_markup (context, "˝");
 		break;
 
-	case GDK_dead_caron:
-		pango_layout_set_text (layout, "ˇ", -1);
+	case GDK_KEY_dead_caron:
+		set_markup (context, "ˇ");
 		break;
 
-	case GDK_dead_cedilla:
-		pango_layout_set_text (layout, "¸", -1);
+	case GDK_KEY_dead_cedilla:
+		set_markup (context, "¸");
 		break;
 
-	case GDK_dead_ogonek:
-		pango_layout_set_text (layout, "˛", -1);
+	case GDK_KEY_dead_ogonek:
+		set_markup (context, "˛");
 		break;
 
-		/* case GDK_dead_iota:
-		 * case GDK_dead_voiced_sound:
-		 * case GDK_dead_semivoiced_sound: */
+		/* case GDK_KEY_dead_iota:
+		 * case GDK_KEY_dead_voiced_sound:
+		 * case GDK_KEY_dead_semivoiced_sound: */
 
-	case GDK_dead_belowdot:
-		pango_layout_set_text (layout, " ̣", -1);
+	case GDK_KEY_dead_belowdot:
+		set_markup (context, " ̣");
 		break;
 
-	case GDK_horizconnector:
-		pango_layout_set_text (layout, "horiz\nconn", -1);
+	case GDK_KEY_horizconnector:
+		set_markup (context, "horiz\nconn");
 		break;
 
-	case GDK_Mode_switch:
-		pango_layout_set_text (layout, "AltGr", -1);
+	case GDK_KEY_Mode_switch:
+		set_markup (context, "AltGr");
 		break;
 
-	case GDK_Multi_key:
-		pango_layout_set_text (layout, "Compose", -1);
+	case GDK_KEY_Multi_key:
+		set_markup (context, "Compose");
 		break;
 
 	default:
 		uc = gdk_keyval_to_unicode (keyval);
 		if (uc != 0 && g_unichar_isgraph (uc)) {
 			buf[g_unichar_to_utf8 (uc, buf)] = '\0';
-			pango_layout_set_text (layout, buf, -1);
+			set_markup (context, buf);
 		} else {
 			gchar *name = gdk_keyval_name (keyval);
-			if (name)
-				pango_layout_set_text (layout, name, -1);
-			else
-				pango_layout_set_text (layout, "", -1);
+			if (name) {
+				set_markup (context, name);
+			} else
+				set_markup (context, "");
 		}
 	}
 }
@@ -991,6 +1002,31 @@ draw_key_label (MatekbdKeyboardDrawingRenderContext * context,
 	}
 }
 
+/**
+   The x offset is calculated for complex shapes. It is the rightmost of the vertical lines in the outline
+ */
+static gint
+calc_origin_offset_x (XkbOutlineRec * outline)
+{
+	gint rv = 0;
+	gint i;
+	XkbPointPtr point = outline->points;
+	if (outline->num_points < 3)
+		return 0;
+	for (i = outline->num_points; --i > 0;) {
+		gint x1 = point->x;
+		gint y1 = point++->y;
+		gint x2 = point->x;
+		gint y2 = point->y;
+
+		/*vertical, bottom to top (clock-wise), on the left */
+		if ((x1 == x2) && (y1 > y2) && (x1 > rv)) {
+			rv = x1;
+		}
+	}
+	return rv;
+}
+
 /* groups are from 0-3 */
 static void
 draw_key (MatekbdKeyboardDrawingRenderContext * context,
@@ -998,6 +1034,8 @@ draw_key (MatekbdKeyboardDrawingRenderContext * context,
 {
 	XkbShapeRec *shape;
 	GdkColor *color;
+	XkbOutlineRec *outline;
+	int origin_offset_x;
 	/* gint i; */
 
 	if (!drawing->xkb)
@@ -1026,9 +1064,9 @@ draw_key (MatekbdKeyboardDrawingRenderContext * context,
 #endif
 
 	/* draw the primary outline */
-	draw_outline (context,
-		      shape->primary ? shape->primary : shape->outlines,
-		      color, key->angle, key->origin_x, key->origin_y);
+	outline = shape->primary ? shape->primary : shape->outlines;
+	draw_outline (context, outline, color, key->angle, key->origin_x,
+		      key->origin_y);
 #if 0
 	/* don't draw other outlines for now, since
 	 * the text placement does not take them into account
@@ -1042,8 +1080,9 @@ draw_key (MatekbdKeyboardDrawingRenderContext * context,
 	}
 #endif
 
+	origin_offset_x = calc_origin_offset_x (outline);
 	draw_key_label (context, drawing, key->keycode, key->angle,
-			key->origin_x, key->origin_y,
+			key->origin_x + origin_offset_x, key->origin_y,
 			shape->bounds.x2, shape->bounds.y2);
 }
 
@@ -1146,7 +1185,7 @@ draw_text_doodad (MatekbdKeyboardDrawingRenderContext * context,
 	y = xkb_to_pixmap_coord (context,
 				 doodad->origin_y + text_doodad->top);
 
-	pango_layout_set_text (context->layout, text_doodad->text, -1);
+	set_markup (context, text_doodad->text);
 	draw_pango_layout (context, drawing, doodad->angle, x, y);
 }
 
@@ -1314,20 +1353,11 @@ create_cairo (MatekbdKeyboardDrawing * drawing)
 	GtkStateType state;
 	if (drawing == NULL)
 		return FALSE;
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (drawing->surface == NULL)
 		return FALSE;
-#else
-	if (drawing->pixmap == NULL)
-		return FALSE;
-#endif
 
 	drawing->renderContext->cr =
-#if GTK_CHECK_VERSION (3, 0, 0)
 	    cairo_create (drawing->surface);
-#else
-	    gdk_cairo_create (GDK_DRAWABLE (drawing->pixmap));
-#endif
 
 	state = gtk_widget_get_state (GTK_WIDGET (drawing));
 	drawing->renderContext->dark_color =
@@ -1355,15 +1385,12 @@ draw_keyboard (MatekbdKeyboardDrawing * drawing)
 
 	gtk_widget_get_allocation (GTK_WIDGET (drawing), &allocation);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	drawing->surface =
-	    gdk_window_create_similar_surface (gtk_widget_get_window (GTK_WIDGET (drawing)),
-	                                       CAIRO_CONTENT_COLOR, allocation.width, allocation.height);
-#else
-	drawing->pixmap =
-	    gdk_pixmap_new (gtk_widget_get_window (GTK_WIDGET (drawing)),
-			    allocation.width, allocation.height, -1);
-#endif
+	    gdk_window_create_similar_surface (gtk_widget_get_window
+					       (GTK_WIDGET (drawing)),
+					       CAIRO_CONTENT_COLOR,
+					       allocation.width,
+					       allocation.height);
 
 	if (create_cairo (drawing)) {
 	        /* blank background */
@@ -1409,61 +1436,28 @@ free_render_context (MatekbdKeyboardDrawing * drawing)
 }
 
 static gboolean
-expose_event (GtkWidget * widget,
 #if GTK_CHECK_VERSION (3, 0, 0)
-	      cairo_t *cr,
+draw (GtkWidget *widget,
+      cairo_t *cr,
+      MatekbdKeyboardDrawing *drawing)
 #else
-	      GdkEventExpose * event,
+expose_event (GtkWidget *widget,
+	      GdkEventExpose *event,
+	      MatekbdKeyboardDrawing *drawing)
 #endif
-	      MatekbdKeyboardDrawing * drawing)
 {
-	GtkAllocation allocation;
-
 #if !GTK_CHECK_VERSION (3, 0, 0)
-        cairo_t *cr;
+        cairo_t *cr = gdk_cairo_create (event->window);
 #endif
 
 	if (!drawing->xkb)
 		return FALSE;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (drawing->surface == NULL)
-#else
-	if (drawing->pixmap == NULL)
-#endif
 		return FALSE;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-        cairo_set_source_surface (cr, drawing->surface, 0, 0);
-        cairo_paint (cr);
-#else
-        cr = gdk_cairo_create (event->window);
-        gdk_cairo_region (cr, event->region);
-        cairo_clip (cr);
-
-        gdk_cairo_set_source_pixmap (cr, drawing->pixmap, 0, 0);
-        cairo_paint (cr);
-
-        cairo_destroy (cr);
-#endif
-
-	if (gtk_widget_has_focus (widget)) {
-		gtk_widget_get_allocation (widget, &allocation);
-		gtk_paint_focus (gtk_widget_get_style (widget),
-#if GTK_CHECK_VERSION (3, 0, 0)
-				 cr,
-#else
-				 gtk_widget_get_window (widget),
-#endif
-				 gtk_widget_get_state (widget),
-#if GTK_CHECK_VERSION (3, 0, 0)
-				 widget, "keyboard-drawing",
-#else
-				 &event->area, widget, "keyboard-drawing",
-#endif
-				 0, 0, allocation.width,
-				 allocation.height);
-	}
+	cairo_set_source_surface (cr, drawing->surface, 0, 0);
+	cairo_paint (cr);
 
 	return FALSE;
 }
@@ -1523,17 +1517,10 @@ size_allocate (GtkWidget * widget,
 {
 	MatekbdKeyboardDrawingRenderContext *context = drawing->renderContext;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (drawing->surface) {
 		cairo_surface_destroy (drawing->surface);
 		drawing->surface = NULL;
 	}
-#else
-	if (drawing->pixmap) {
-		g_object_unref (drawing->pixmap);
-		drawing->pixmap = NULL;
-	}
-#endif
 
 	if (!context_setup_scaling (context, drawing,
 				    allocation->width, allocation->height,
@@ -2034,13 +2021,9 @@ destroy (MatekbdKeyboardDrawing * drawing)
 		drawing->idle_redraw = 0;
 	}
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	if (drawing->surface != NULL) {
 		cairo_surface_destroy (drawing->surface);
 	}
-#else
-	g_object_unref (drawing->pixmap);
-#endif
 }
 
 static void
@@ -2077,11 +2060,7 @@ matekbd_keyboard_drawing_init (MatekbdKeyboardDrawing * drawing)
 		drawing->screen_num =
 		    gdk_screen_get_number (gdk_screen_get_default ());
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	drawing->surface = NULL;
-#else
-	drawing->pixmap = NULL;
-#endif
 	alloc_render_context (drawing);
 
 	drawing->keyboard_items = NULL;
@@ -2108,7 +2087,7 @@ matekbd_keyboard_drawing_init (MatekbdKeyboardDrawing * drawing)
 
 	XkbGetNames (drawing->display, XkbAllNamesMask, drawing->xkb);
 	drawing->l3mod = XkbKeysymToModifiers (drawing->display,
-					       GDK_ISO_Level3_Shift);
+					       GDK_KEY_ISO_Level3_Shift);
 
 	drawing->xkbOnDisplay = TRUE;
 
@@ -2144,10 +2123,11 @@ matekbd_keyboard_drawing_init (MatekbdKeyboardDrawing * drawing)
 			       | GDK_FOCUS_CHANGE_MASK);
 #if GTK_CHECK_VERSION (3, 0, 0)
 	g_signal_connect (G_OBJECT (drawing), "draw",
+			  G_CALLBACK (draw), drawing);
 #else
 	g_signal_connect (G_OBJECT (drawing), "expose-event",
-#endif
 			  G_CALLBACK (expose_event), drawing);
+#endif
 	g_signal_connect_after (G_OBJECT (drawing), "key-press-event",
 				G_CALLBACK (key_event), drawing);
 	g_signal_connect_after (G_OBJECT (drawing), "key-release-event",
@@ -2233,7 +2213,7 @@ matekbd_keyboard_drawing_set_mods (MatekbdKeyboardDrawing * drawing, guint mods)
 
 /**
  * matekbd_keyboard_drawing_render:
- * @drawing: keyboard layout to render
+ * @kbdrawing: keyboard layout to render
  * @cr:        Cairo context to render to
  * @layout:    Pango layout to use to render text
  * @x:         left coordinate (pixels) of region to render in
@@ -2248,7 +2228,7 @@ matekbd_keyboard_drawing_set_mods (MatekbdKeyboardDrawing * drawing, guint mods)
  * @returns:   %TRUE on success, %FALSE on failure
  */
 gboolean
-matekbd_keyboard_drawing_render (MatekbdKeyboardDrawing * drawing,
+matekbd_keyboard_drawing_render (MatekbdKeyboardDrawing * kbdrawing,
 			      cairo_t * cr,
 			      PangoLayout * layout,
 			      double x, double y,
@@ -2257,22 +2237,22 @@ matekbd_keyboard_drawing_render (MatekbdKeyboardDrawing * drawing,
 {
 	MatekbdKeyboardDrawingRenderContext context = {
 		cr,
-		drawing->renderContext->angle,
+		kbdrawing->renderContext->angle,
 		layout,
 		pango_font_description_copy (gtk_widget_get_style
 					     (GTK_WIDGET
-					      (drawing))->font_desc),
+					      (kbdrawing))->font_desc),
 		1, 1,
-		&gtk_widget_get_style (GTK_WIDGET (drawing))->dark
-		    [gtk_widget_get_state (GTK_WIDGET (drawing))]
+		&gtk_widget_get_style (GTK_WIDGET (kbdrawing))->dark
+		    [gtk_widget_get_state (GTK_WIDGET (kbdrawing))]
 	};
 
-	if (!context_setup_scaling (&context, drawing, width, height,
+	if (!context_setup_scaling (&context, kbdrawing, width, height,
 				    dpi_x, dpi_y))
 		return FALSE;
 	cairo_translate (cr, x, y);
 
-	draw_keyboard_to_context (&context, drawing);
+	draw_keyboard_to_context (&context, kbdrawing);
 
 	pango_font_description_free (context.font_desc);
 
@@ -2541,11 +2521,6 @@ show_layout_response (GtkWidget * dialog, gint resp)
 	const gchar *groupName;
 
 	switch (resp) {
-	case GTK_RESPONSE_HELP:
-		gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)),
-			      "ghelp:gswitchit?layout-view",
-			      gtk_get_current_event_time (), NULL);
-		return;
 	case GTK_RESPONSE_CLOSE:
 		gtk_window_get_position (GTK_WINDOW (dialog), &rect.x,
 					 &rect.y);
@@ -2685,11 +2660,7 @@ matekbd_keyboard_drawing_new_dialog (gint group, gchar * group_name)
 
 	g_object_set_data (G_OBJECT (dialog), "kbdraw", kbdraw);
 
-	#if GTK_CHECK_VERSION(3, 0, 0)
-		g_signal_connect_swapped(G_OBJECT(dialog), "destroy", G_CALLBACK(g_object_unref),  g_object_get_data(G_OBJECT(dialog), "builderData"));
-	#else
-		g_signal_connect_swapped(GTK_OBJECT(dialog), "destroy", G_CALLBACK(g_object_unref),  g_object_get_data(G_OBJECT(dialog), "builderData"));
-	#endif
+		g_signal_connect_swapped(dialog, "destroy", G_CALLBACK(g_object_unref),  g_object_get_data(G_OBJECT(dialog), "builderData"));
 
 	gtk_widget_show_all(dialog);
 
