@@ -16,6 +16,7 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#include <config.h>
 
 #include <memory.h>
 
@@ -23,7 +24,7 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gprintf.h>
 
 #include <matekbd-status.h>
@@ -826,6 +827,10 @@ matekbd_status_get_xkl_engine ()
 	return globals.engine;
 }
 
+/**
+ * matekbd_status_get_group_names:
+ * Returns: (transfer none) (array zero-terminated=1): List of group names
+ */
 gchar **
 matekbd_status_get_group_names ()
 {
